==> react-view-router-2.1.2.tgz: package/es/router-view.js <==
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { renderRoute, normalizeRoute, normalizeRoutes, isFunction, isRouteChanged, isRoutesChanged, isPropChanged, getHostRouterView } from './util';

class RouterView extends React.Component {
  constructor(props) {
    super(props);

    _defineProperty(this, "_isMounted", void 0);

    _defineProperty(this, "target", void 0);

    _defineProperty(this, "_reactInternalFiber", void 0);

    const router = props && props.router;
    const depth = props && props.depth ? Number(props.depth) : 0;
    const state = {
      _routerView: this,
      _routerRoot: true,
      _routerParent: null,
      _routerDepth: depth,
      _routerInited: false,
      _routerResolving: false,
      router,
      parentRoute: null,
      currentRoute: null,
      routes: router ? this._filterRoutes(router.routes) : []
    };
    this._isMounted = false;
    this.state = state;
    this.target = new.target;
    this._updateRef = this._updateRef.bind(this);
    this._filterRoutes = this._filterRoutes.bind(this);
  }

  get name() {
    let name = this.props.name;
    if (!name) return 'default';
    return name;
  }

  get currentRef() {
    let currentRoute = this.state.currentRoute;
    return currentRoute && currentRoute.componentInstances[this.name];
  }

  _updateRef(ref) {
    let currentRoute = this.state.currentRoute;
    if (currentRoute) currentRoute.componentInstances[this.name] = ref;
    if (this.props && this.props._updateRef) this.props._updateRef(ref);
    if (this._isMounted) this.setState({
      currentRoute
    });
  }

  _filterRoutes(routes, state) {
    const {
      name,
      filter
    } = this.props;
    let ret = routes && routes.filter(r => {
      if (r.config) r = r.config;
      const hasName = name && name !== 'default';
      if (r.redirect || r.index) return hasName ? name === r.name : !r.name;
      return hasName ? r.components && r.components[name] : r.component || r.components && r.components.default;
    });
    if (filter) ret = filter(ret, state || this.state);
    return ret;
  }

  _getRouteMatch(state, depth = 0) {
    if (!state) state = this.state;
    const matched = state.router && state.router.currentRoute && state.router.currentRoute.matched || [];
    return matched.length > depth ? matched[depth] : null;
  }

  _refreshCurrentRoute(state, newState, callback) {
    if (!state) state = this.state;
    const router = state.router;
    if (!router) throw new Error('state.router is null!');

    let currentRoute = this._getRouteMatch(state, state._routerDepth);

    if (!currentRoute) {
      currentRoute = router.createMatchedRoute(normalizeRoute({
        path: ''
      }, state.parentRoute, state._routerDepth), state.parentRoute);
      router.currentRoute && router.currentRoute.matched.push(currentRoute);
    } else if (!currentRoute || currentRoute.redirect) currentRoute = null;

    if (currentRoute) currentRoute.viewInstances[this.name] = this;

    if (this.state && this.state._routerInited) {
      if (newState) Object.assign(newState, {
        currentRoute
      });else if (this._isMounted) {
        if (router._isVuelikeComponent(this.currentRef) && isRouteChanged(this.state.currentRoute, currentRoute)) {
          this.currentRef._willUnmount && this.currentRef._willUnmount();
        }

        this.setState({
          currentRoute
        });
      }
    }

    if (this._isMounted) callback && callback();
    return currentRoute;
  }

  _updateResolving(resolving) {
    if (this._isMounted) this.setState({
      _routerResolving: Boolean(resolving)
    });
  }

  _resolveFallback() {
    let ret = null;
    let fallback = this.props.fallback;

    if (isFunction(fallback)) {
      ret = fallback({
        parentRoute: this.state.parentRoute,
        currentRoute: this.state.currentRoute,
        inited: this.state._routerInited,
        resolving: this.state._routerResolving,
        depth: this.state._routerDepth
      });
    } else if (React.isValidElement(fallback)) ret = fallback;

    return ret || null;
  }

  isNull(route) {
    return !route || !route.path || route.subpath === '';
  }

  async componentDidMount() {
    this._isMounted = true;
    if (this.state._routerInited) return;

    const state = _objectSpread({}, this.state);

    if (state._routerRoot && state.router) {
      const router = state.router;
      router.viewRoot = this; // if (!router.parent) {
      //   let parent = getHostRouterView(this);
      //   router.parent = (parent && parent.state.router) || null;
      // }

      const pendingRoute = router.pendingRoute;
      router.pendingRoute = null;

      router._handleRouteInterceptor(pendingRoute || router.history.location, (ok, to) => {
        if (!ok) return;
        router && to && router.updateRoute(to);
        state.currentRoute = this._refreshCurrentRoute();
        if (this._isMounted) this.setState(Object.assign(state, {
          _routerInited: this._isMounted
        }));
      }, true);

      return;
    }

    if (!this._reactInternalFiber) return;
    let parent = getHostRouterView(this);

    if (parent) {
      state._routerRoot = false;
      state._routerParent = parent.state._routerView;
      if (!state.router) state.router = parent.state.router;
      state._routerDepth = parent.state._routerDepth + 1;
    }

    if (state._routerDepth) {
      state.parentRoute = this._getRouteMatch(state, state._routerDepth - 1);
      state.routes = state.parentRoute ? this._filterRoutes(state.parentRoute.config.children) : [];
      state.currentRoute = this._refreshCurrentRoute(state);
    } else console.error('[RouterView] cannot find root RouterView instance!', this);

    if (this._isMounted) this.setState(Object.assign(state, {
      _routerInited: true
    }));
  }

  componentWillUnmount() {
    this._isMounted = false;
    const {
      _routerRoot,
      router
    } = this.state;
    _routerRoot && router && (router.viewRoot = null);
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (!this._isMounted) return false;
    if (this.state._routerResolving !== nextState._routerResolving) return true;
    if (this.state._routerInited !== nextState._routerInited) return true;
    if (this.state._routerDepth !== nextState._routerDepth) return true;
    if (this.state.router !== nextState.router) return true;
    if (isPropChanged(this.props, nextProps)) return true;
    if (isRouteChanged(this.state.currentRoute, nextState.currentRoute)) return true;
    if (isRoutesChanged(this.state.routes, nextState.routes)) return true;
    return false;
  }

  push(...routes) {
    const state = _objectSpread({}, this.state);

    state.routes.push(...normalizeRoutes(routes, state.parentRoute));
    if (this._isMounted) this.setState(state);
    return state.routes;
  }

  splice(idx, len, ...routes) {
    const state = _objectSpread({}, this.state);

    state.routes.splice(idx, len, ...normalizeRoutes(routes, state.parentRoute));
    if (this._isMounted) this.setState(state);
    return state.routes;
  }

  indexOf(route) {
    if (typeof route === 'string') route = {
      path: route
    };
    const {
      routes
    } = this.state;
    return routes.findIndex(r => r.path === route.path);
  }

  remove(route) {
    if (typeof route === 'string') route = {
      path: route
    };
    const {
      routes
    } = this.state;
    const index = this.indexOf(route);
    if (~index) routes.splice(index, 1);
    if (this._isMounted) this.setState({
      routes
    });
    return ~index ? route : undefined;
  }

  getComponent(currentRoute, excludeProps) {
    if (!currentRoute) return null;
    const {
      routes,
      router
    } = this.state;

    const _this$props = this.props,
          {
      container,
      children
    } = _this$props,
          props = _objectWithoutProperties(_this$props, ["container", "children"]);

    const {
      query = {},
      params
    } = router && router.currentRoute || {};
    const targetExcludeProps = this.target.defaultProps.excludeProps || RouterView.defaultProps.excludeProps || [];
    (excludeProps || targetExcludeProps).forEach(key => delete props[key]);
    return renderRoute(currentRoute, routes, props, children, {
      name: this.name,
      query,
      params,
      container,
      ref: this._updateRef
    });
  }

  renderCurrent(currentRoute) {
    if (this.isNull(currentRoute)) return this.props.children || null;
    return this.getComponent(currentRoute);
  }

  render() {
    if (!this.state._routerInited) return this._resolveFallback();
    let ret = this.renderCurrent(this.state.currentRoute);

    if (this.state._routerResolving) {
      ret = React.createElement(React.Fragment, {}, ret, this._resolveFallback());
    }

    return ret;
  }

}

_defineProperty(RouterView, "defaultProps", void 0);

RouterView.defaultProps = {
  excludeProps: ['_updateRef', 'router', 'excludeProps', 'beforeEach', 'beforeResolve', 'afterEach', 'fallback']
};
export { RouterView as RouterViewComponent };
export default React.forwardRef((props, ref) => React.createElement(RouterView, _objectSpread({}, props, {
  _updateRef: ref
})));

==> react-view-router-2.1.3.tgz: package/es/router-view.js <==
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { renderRoute, normalizeRoute, normalizeRoutes, isFunction, isRouteChanged, isRoutesChanged, isPropChanged, getHostRouterView, warn } from './util';
import { computeRootMatch } from './match-path';
import { RouterContext, RouterViewContext } from './context';

class RouterView extends React.Component {
  constructor(props) {
    super(props);

    _defineProperty(this, "_isMounted", void 0);

    _defineProperty(this, "target", void 0);

    _defineProperty(this, "_reactInternalFiber", void 0);

    const router = props && props.router;
    const depth = props && props.depth ? Number(props.depth) : 0;
    const state = {
      _routerView: this,
      _routerRoot: true,
      _routerParent: null,
      _routerDepth: depth,
      _routerInited: false,
      _routerResolving: false,
      router,
      parentRoute: null,
      currentRoute: null,
      routes: router ? this._filterRoutes(router.routes) : []
    };
    this._isMounted = false;
    this.state = state;
    this.target = new.target;
    this._updateRef = this._updateRef.bind(this);
    this._filterRoutes = this._filterRoutes.bind(this);
  }

  get name() {
    let name = this.props.name;
    if (!name) return 'default';
    return name;
  }

  get currentRef() {
    let currentRoute = this.state.currentRoute;
    return currentRoute && currentRoute.componentInstances[this.name];
  }

  _updateRef(ref) {
    let currentRoute = this.state.currentRoute;
    if (currentRoute) currentRoute.componentInstances[this.name] = ref;
    if (this.props && this.props._updateRef) this.props._updateRef(ref); // if (this._isMounted) this.setState({ currentRoute });
  }

  _filterRoutes(routes, state) {
    const {
      name,
      filter
    } = this.props;
    let ret = routes && routes.filter(r => {
      if (r.config) r = r.config;
      const hasName = name && name !== 'default';
      if (r.redirect || r.index) return hasName ? name === r.name : !r.name;
      return hasName ? r.components && r.components[name] : r.component || r.components && r.components.default;
    });
    if (filter) ret = filter(ret, state || this.state);
    return ret;
  }

  _getRouteMatch(state, depth = 0) {
    if (!state) state = this.state;
    const matched = state.router && state.router.currentRoute && state.router.currentRoute.matched || [];
    return matched.length > depth ? matched[depth] : null;
  }

  _refreshCurrentRoute(state, newState, callback) {
    if (!state) state = this.state;
    const router = state.router;
    if (!router) throw new Error('state.router is null!');

    let currentRoute = this._getRouteMatch(state, state._routerDepth);

    if (!currentRoute) {
      let route = normalizeRoute({
        path: ''
      }, state.parentRoute && state.parentRoute.config);
      currentRoute = router.createMatchedRoute(route, computeRootMatch());
      router.currentRoute && router.currentRoute.matched.push(currentRoute);
    } else if (!currentRoute || currentRoute.redirect) currentRoute = null;

    if (currentRoute) {
      currentRoute.viewInstances[this.name] = this;
    }

    if (this.state && this.state._routerInited) {
      if (newState) Object.assign(newState, {
        currentRoute
      });else if (this._isMounted) {
        if (router._isVuelikeComponent(this.currentRef) && isRouteChanged(this.state.currentRoute, currentRoute)) {
          this.currentRef._willUnmount && this.currentRef._willUnmount();
        }

        try {
          this.setState({
            currentRoute
          });
        } catch (ex) {
          console.error(ex);
        }
      }
    }

    if (this._isMounted) callback && callback();
    return currentRoute;
  }

  _updateResolving(resolving) {
    if (this._isMounted) this.setState({
      _routerResolving: Boolean(resolving)
    });
  }

  _resolveFallback() {
    let ret = null;
    let fallback = this.props.fallback;

    if (isFunction(fallback)) {
      ret = fallback({
        parentRoute: this.state.parentRoute,
        currentRoute: this.state.currentRoute,
        inited: this.state._routerInited,
        resolving: this.state._routerResolving,
        depth: this.state._routerDepth
      });
    } else if (React.isValidElement(fallback)) ret = fallback;

    return ret || null;
  }

  isNull(route) {
    return !route || !route.path || route.subpath === '';
  }

  async componentDidMount() {
    this._isMounted = true;
    if (this.state._routerInited) return;

    const state = _objectSpread({}, this.state);

    if (state._routerRoot && state.router) {
      const router = state.router;
      router.viewRoot = this; // if (!router.parent) {
      //   let parent = getHostRouterView(this);
      //   router.parent = (parent && parent.state.router) || null;
      // }

      const pendingRoute = router.pendingRoute;
      router.pendingRoute = null;

      if (!router.isRunning) {
        warn('[react-view-router] warning: router is not running in RouterView.componentDidMount');
      }

      const [, location] = router.history.getIndexAndLocation ? router.history.getIndexAndLocation() : [router.history.index, router.history.location];

      router._handleRouteInterceptor(pendingRoute || _objectSpread({}, location), (ok, to) => {
        if (!ok) return;
        router && to && router.updateRoute(to);
        state.currentRoute = this._refreshCurrentRoute();
        if (this._isMounted) this.setState(Object.assign(state, {
          _routerInited: this._isMounted
        }));
        if (isFunction(ok)) ok(true);
      }, true);

      return;
    }

    if (!this._reactInternalFiber) return;
    let parent = getHostRouterView(this);

    if (parent) {
      state._routerRoot = false;
      state._routerParent = parent.state._routerView;
      if (!state.router) state.router = parent.state.router;
      state._routerDepth = parent.state._routerDepth + 1;
    }

    if (state._routerDepth) {
      state.parentRoute = this._getRouteMatch(state, state._routerDepth - 1);
      state.routes = state.parentRoute ? this._filterRoutes(state.parentRoute.config.children) : [];
      state.currentRoute = this._refreshCurrentRoute(state);
    } else console.error('[RouterView] cannot find root RouterView instance!', this);

    if (this._isMounted) this.setState(Object.assign(state, {
      _routerInited: true
    }));
  }

  componentWillUnmount() {
    this._isMounted = false;
    const {
      _routerRoot,
      router
    } = this.state;
    _routerRoot && router && (router.viewRoot = null);
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (!this._isMounted) return false;
    if (this.state._routerResolving !== nextState._routerResolving) return true;
    if (this.state._routerInited !== nextState._routerInited) return true;
    if (this.state._routerDepth !== nextState._routerDepth) return true;
    if (this.state.router !== nextState.router) return true;
    if (isPropChanged(this.props, nextProps)) return true;
    if (isRouteChanged(this.state.currentRoute, nextState.currentRoute)) return true;
    if (isRoutesChanged(this.state.routes, nextState.routes)) return true;
    return false;
  }

  push(...routes) {
    const state = _objectSpread({}, this.state);

    state.routes.push(...normalizeRoutes(routes, state.parentRoute && state.parentRoute.config));
    if (this._isMounted) this.setState(state);
    return state.routes;
  }

  splice(idx, len, ...routes) {
    const state = _objectSpread({}, this.state);

    state.routes.splice(idx, len, ...normalizeRoutes(routes, state.parentRoute && state.parentRoute.config));
    if (this._isMounted) this.setState(state);
    return state.routes;
  }

  indexOf(route) {
    if (typeof route === 'string') route = {
      path: route
    };
    const {
      routes
    } = this.state;
    return routes.findIndex(r => r.path === route.path);
  }

  remove(route) {
    if (typeof route === 'string') route = {
      path: route
    };
    const {
      routes
    } = this.state;
    const index = this.indexOf(route);
    if (~index) routes.splice(index, 1);
    if (this._isMounted) this.setState({
      routes
    });
    return ~index ? route : undefined;
  }

  getComponent(currentRoute, excludeProps) {
    if (!currentRoute) return null;
    const {
      routes,
      router
    } = this.state;

    const _this$props = this.props,
          {
      container,
      children
    } = _this$props,
          props = _objectWithoutProperties(_this$props, ["container", "children"]);

    const {
      query = {},
      params
    } = router && router.currentRoute || {};
    const targetExcludeProps = this.target.defaultProps.excludeProps || RouterView.defaultProps.excludeProps || [];
    (excludeProps || targetExcludeProps).forEach(key => delete props[key]);
    return renderRoute(currentRoute, routes, props, children, {
      router,
      name: this.name,
      query,
      params,
      container,
      ref: this._updateRef
    });
  }

  renderCurrent(currentRoute) {
    if (this.isNull(currentRoute)) return this.props.children || null;
    return this.getComponent(currentRoute);
  }

  render() {
    if (!this.state._routerInited) return this._resolveFallback();
    let ret = React.createElement(RouterViewContext.Provider, {
      value: this
    }, this.renderCurrent(this.state.currentRoute));

    if (this.state._routerRoot) {
      ret = React.createElement(RouterContext.Provider, {
        value: this.state.router
      }, ret);
    }

    if (this.state._routerResolving) {
      ret = React.createElement(React.Fragment, {}, ret, this._resolveFallback());
    }

    return ret;
  }

}

_defineProperty(RouterView, "defaultProps", void 0);

const RouterViewWrapper = React.forwardRef((props, ref) => React.createElement(RouterView, _objectSpread(_objectSpread({}, props), {}, {
  _updateRef: ref
})));
RouterView.defaultProps = {
  excludeProps: ['_updateRef', 'router', 'excludeProps', 'beforeEach', 'beforeResolve', 'afterEach', 'fallback']
};
export { RouterViewWrapper, RouterView as RouterViewComponent };
export default RouterViewWrapper;
